"""

app.app
~~~~~~~~~~

Manages the configuration and creation process for the App.

"""
import logging
import random
import os

from flask import Flask, g, render_template

from app import models, views
from app.extensions import db, login_manager


def create_app():
    app = Flask('app')
    config = os.getenv('CONFIG', 'app.config.DevelopmentConfig')
    app.config.from_object(config)

    #TODO: Flask Login properly
    configure_blueprints(app)
    configure_extensions(app)
    configure_template_filters(app)
    configure_context_processors(app)
    configure_before_handlers(app)
    configure_error_handlers(app)
    configure_logging(app)

    return app


def configure_blueprints(app):
    """Configures the Blueprints"""

    app.register_blueprint(views.book, url_prefix=app.config['BOOK_URL_PREFIX'])
    app.register_blueprint(views.image, url_prefix=app.config['IMAGE_URL_PREFIX'])
    app.register_blueprint(views.index, url_prefix=app.config['INDEX_URL_PREFIX'])
    app.register_blueprint(views.library, url_prefix=app.config['LIBRARY_URL_PREFIX'])
    app.register_blueprint(views.listing, url_prefix=app.config['LISTING_URL_PREFIX'])
    app.register_blueprint(views.message, url_prefix=app.config['MESSAGE_URL_PREFIX'])
    app.register_blueprint(views.notification, url_prefix=app.config['NOTIFICATION_URL_PREFIX'])
    app.register_blueprint(views.transaction, url_prefix=app.config['TRANSACTION_URL_PREFIX'])
    app.register_blueprint(views.user, url_prefix=app.config['USER_URL_PREFIX'])
    app.register_blueprint(views.interested_list, url_prefix=app.config['INTERESTED_LIST_PREFIX'])

def configure_extensions(app):
    """Configures the extensions."""
    # Flask-SQLAlchemy
    db.init_app(app)

    # Flask-Login
    login_manager.login_view = app.config['LOGIN_VIEW']
    login_manager.refresh_view = app.config['REAUTH_VIEW']
    # login_manager.anonymous_user = Guest

    @login_manager.user_loader
    def load_user(user_id):
        """Loads the user. Required by the Flask-Login extension."""
        user_instance = User.query.get(user_id)

        if user_instance:
            return user_instance
        else:
            return None

    login_manager.init_app(app)

def configure_template_filters(app):
    """Configures the template filters."""
    filters = {}

    """
    def example(value):
        pass

    filters['example'] = example
    """

    app.jinja_env.filters.update(filters)


def configure_context_processors(app):
    """Configures the context processors."""
    @app.context_processor
    def inject_version():
        return dict(version=app.config['VERSION'])


def configure_before_handlers(app):
    """Configures the before request handlers."""

    # @app.before_request
    # def load_user_agents():
    #     uas = app.config["USER_AGENT_LIST"]
    #     random.shuffle(uas)
    #     g.user_agent_list = uas


def configure_error_handlers(app):
    """Configures the error handlers."""

    @app.errorhandler(403)
    def forbidden_page(error):
        return render_template("errors/403.html"), 403

    @app.errorhandler(404)
    def page_not_found(error):
        return render_template("errors/404.html"), 404

    @app.errorhandler(500)
    def server_error_page(error):
        return render_template("errors/500.html"), 500


def configure_logging(app):
    """Configures logging."""
    handler = logging.StreamHandler()
    handler.setLevel(logging.DEBUG)
    app.logger.addHandler(handler)
