"""
listing.py

Represents a listing.

Created by Stephen Andrews, September 19th, 2018.
"""

import maya
from sqlalchemy.orm import relationship
from sqlalchemy.types import String

from app.extensions import AuditMixin, db
from .book import Book
from .user import User


class Listing(db.Model, AuditMixin):
    """The model for a listing."""
    __tablename__ = 'listing'

    id = db.Column(db.Integer, primary_key=True)
    seller_id = db.Column(db.Integer, db.ForeignKey('user.id'), nullable=False)
    book_id = db.Column(db.Integer, db.ForeignKey('book.id'), nullable=False)
    condition = db.Column(db.Text, nullable=False)
    description = db.Column(db.Text)
    photos = db.Column(db.ARRAY(String))
    visible = db.Column(db.Boolean, server_default='t', default=True, nullable=False)

    book = relationship('Book', primaryjoin='Listing.book_id == Book.id')
    sale = relationship('Sale', primaryjoin='Listing.id == Sale.listing_id')
    seller = relationship('User', primaryjoin='Listing.seller_id == User.id')


    def __repr__(self):
        representation = '<Listing {} - Seller ID: {}, Book ID: {}, Condition: {} Description: {} Photos: {}>'
        return representation.format(self.id, self.seller_id, self.book_id, self.condition, self.description, self.photos)

    def serialize(self, detailed=True):
        """
        Return a serialized version of a listing.

        :param detailed: Whether or not to show specific details such as condition, description, and price.
                         Defaults to true.
        :return: A dictionary representing the listing.
        """
        j = {}
        j['listing_id'] = self.id
        j['seller'] = User.query.filter_by(id = self.seller_id).first().serialize()
        j['book'] = Book.query.filter_by(id = self.book_id).first().serialize()
        j['photos'] = self.photos
        j['visible'] = self.visible
        j['created'] = self.created
        j['slang_created'] = maya.parse(self.created).slang_time()
        if detailed:
            j['condition'] = self.condition
            j['description'] = self.description

            if self.sale:
                j['price'] = self.sale[0].price
                j['type'] = 'SALE'

        return j
