"""
book.py

API endpoints for interacting with a book.

Created by Stephen Andrews, September 18th, 2018.
"""

import json
import re
from typing import List

import requests
from flask import Blueprint, jsonify, request

from app.extensions import db
from app.models import Author, Book
from discovery.google_books_api import GoogleBooks
from app.utils import verify_session


book = Blueprint('book', __name__)
book_api = GoogleBooks()

# Base URL of Google books API
API_BASE = 'https://www.googleapis.com/books/v1/volumes?q={type}:{search_term}'

@book.before_request
def authorize():
    """Require authorization for all views."""
    if not request.headers.get('authorization'):
        return 'unauthorized', 401
    temp = request.headers['authorization'].split(' ')
    if not len(temp) == 2:
        return 'invalid authorization header'
    if not verify_session(temp[1]):
        return 'unauthorized', 401


@book.route('/')
@book.route('/<string:isbn>')
def get(isbn=None) -> dict:
    """
    Get a book by its ISBN.

    :param isbn: Either an ISBN-10 or ISBN-13.
    :return: A serialized book.
    """
    if isbn:
        return jsonify(get_book_by_isbn(isbn).serialize())

    return jsonify([b.serialize() for b in Book.query.all()])

@book.route('/search/<string:search_term>')
def search(search_term: str) -> dict:
    """
    Search endpoint used for typeahead suggestions.

    :param search_term: The search term to fuzzy match with.
    :return: A dictionary containing the found books.
    """

    return jsonify(book_api.suggest(search_term))

def get_book_by_isbn(isbn: str) -> Book:
    """
    Get a book from the database for a given ISBN.

    :param isbn: Either an ISBN-10 or ISBN-13 of the desired book.
    :return: A book model.
    """
    ISBN_10 = 10
    ISBN_13 = 13

    book = None
    if len(isbn) == ISBN_13:
        book = Book.query.filter_by(isbn_13=isbn).first()
    elif len(isbn) == ISBN_10:
        book = Book.query.filter_by(isbn_10=isbn).first()

    # Book already exists in database
    if book:
        return book

    book_data = book_api.search_isbn(isbn)
    book = insert_book(book_data)

    return book

def insert_authors(author_data: dict) -> List[Author]:
    """
    Insert an author into the database.

    :param authors: A list of authors.
    :return: The author model that was created.
    """
    authors = []
    for author in author_data:
        data = author.split(' ')

        # Some results have a middle name included
        # TODO: Might want to adjust our data model to capture this
        first, last = ('', '')
        if len(data) == 1: # No first name provided
            last = data[0]
            first = ''
        elif len(data) == 2:
            first, last = data
        elif len(data) == 3:
            first, last = (data[0], data[2])

        author = Author.query.filter_by(first_name=first, last_name=last).first()
        if not author:
            author = Author(first_name=first, last_name=last)

        authors.append(author)
        db.session.add(author)
        db.session.commit()

    return authors

def insert_book(data: dict) -> Book:
    """
    Insert a book into the database.

    :param data: A dictionary containing required data to create a book.
    :return: The book model that was created.
    """
    print('insert_book: {}'.format(data))
    authors = insert_authors(data.pop('authors'))
    book = Book(**data)
    book.authors = authors

    db.session.add(book)
    db.session.commit()

    return book
