"""
listing.py

API endpoints for interacting with a listing.

Created by Stephen Andrews, September 19th, 2018.
"""

from typing import List

import requests
import psycopg2
from flask import Blueprint, jsonify, request
from sqlalchemy import and_, desc

from app.extensions import db
from app.models import Listing, Sale, Book
from util.logger import Logger

from app.utils import verify_session

listing = Blueprint('listing', __name__)
logger = Logger(logfile_name= 'listing_view')

@listing.before_request
def authorize():
    """Require authorization for all views."""
    if not request.headers.get('authorization'):
        return 'unauthorized', 401
    temp = request.headers['authorization'].split(' ')
    if not len(temp) >= 2:
        return 'invalid authorization header'
    if not verify_session(temp[1]):
        return 'unauthorized', 401

@listing.route('/')
@listing.route('/&p=<int:page_number>')
@listing.route('/&p=<int:page_number>&c=<int:count>')
@listing.route('/<int:book_id>')
@listing.route('/<int:book_id>&p=<int:page_number>')
@listing.route('/<int:book_id>&p=<int:page_number>&c=<int:count>')
def get(book_id=None, page_number=1, count=25):
    """
    Get all listings.

    :param book_id: Optional parameter to get listings only for a particular book.
    :return: A json response containing the active listings.
    """
    # This is to display individual postings for a specified book
    if book_id:
        query = Listing.query.filter(and_(
            Listing.book_id==book_id,
            Listing.visible==True
        ))
        listings = apply_filters(request, query).paginate(page_number, count).items
        return jsonify([l.serialize() for l in listings])

    # Show/search distinct books
    query = Listing.query.distinct(Listing.book_id)
    title = request.args.get('title')

    if title:
        query = query.join(Listing.book).filter(and_(
            Book.title.ilike('%{}%'.format(title)),
            Listing.visible==True # redundant? Not sure, but it won't hurt the results, only the performance!
        ))
    
    tempquery = query.filter(Listing.visible==True).paginate(page_number, count).items

    return jsonify([l.serialize(detailed=False) for l in tempquery]) 

@listing.route('/post', methods=['POST'])
def post_listing():
    """Post a new listing."""
    payload = request.json
    if not payload:
        return 'No content received', 400

    # Extract data from POST
    seller_id = payload.get('seller_id', None)
    book_id = payload.get('book_id', None)
    condition = payload.get('condition', None)
    description = payload.get('description', None) # Optional
    price = payload.get('price', None) # Indicates the listing is of type sale
    photos = payload.get('photos', None)

    if not seller_id or not book_id or not condition:
        print('Malformed request: {}'.format(payload))
        return 'Malformed request', 400

    listing = Listing(seller_id=seller_id, book_id=book_id, condition=condition, description=description, photos=photos)
    db.session.add(listing)
    db.session.commit()

    # Sale listing
    if price:
        if not create_sale(listing.id, price):
            return 'Unable to create sale', 404

    return 'Posted new listing: {}'.format(listing), 200

def create_sale(listing_id: int, price: int) -> bool:
    """
    Create a new sale for a given listing id and price.

    :param listing_id: The listing id the sale is associated with.
    :param price: The price of the sale.
    :return: A boolean indicating whether or not the sale was created.
    """
    sale = Sale(listing_id=listing_id, price=price)

    try:
        db.session.add(sale)
        db.session.commit()
    except psycopg2.Error:
        self.logger.error('Unable to create sale...')
        return False

    return True

# remove/hide a listing/sale
@listing.route('/remove/<string:listing_id>', methods=['DELETE'])
def remove_listing(listing_id):
    entry = Listing.query.filter(Listing.id==listing_id).first()

    if not entry:
        return 'Invalid listing id', 404

    entry.visible = False
    db.session.commit()

    return 'Removed listing', 200

def apply_filters(request, query):
    """
    Filter listings by a condition string.

    :param condition: The condition string to apply as a filter.
    :return: A list of listing models.
    """

    condition = request.args.get('condition')
    low_price = request.args.get('low')
    high_price = request.args.get('high')
    order = request.args.get('order')

    if condition:
        query = query.filter_by(condition=condition)
    
    if low_price and high_price:
        query = query.join(Listing.sale).filter(and_(
            Sale.price >= low_price,
            Sale.price <= high_price
        ))
    
    if order:
        if order.lower() == 'new':
            query = query.order_by(desc(Listing.created))

        if order.lower() == 'price':
            query = query.join(Listing.sale).order_by(Sale.price)
    
    return query

