"""
message.py

A view for messaging other users.

Created by Stephen Andrews, October 25th, 2018.
"""

from flask import Blueprint, current_app, jsonify, render_template, request
from sqlalchemy import and_, or_, desc

from app.extensions import db
from app.models import Listing, Message, Thread, Transaction, User
from app.utils import send_push_message
from util.logger import Logger

from app.utils import verify_session

message = Blueprint('message', __name__)
logger = Logger(logfile_name= 'message_view')

@message.before_request
def authorize():
    """Require authorization for all views."""
    if current_app.config['DEBUG']:
        return

    if not request.headers.get('authorization'):
        return 'unauthorized', 401
    temp = request.headers['authorization'].split(' ')
    if not len(temp) == 2:
        return 'invalid authorization header'
    if not verify_session(temp[1]):
        return 'unauthorized', 401

@message.route('/<int:user_id>')
def get(user_id):
    """
    Get all threads for a given user.

    :param user_id: The id of the user to get a list of threads for.
    """
    threads = Thread.query.filter(or_(
        Thread.from_user == user_id,
        Thread.to_user == user_id
    )).all()

    return jsonify([t.serialize() for t in threads])

@message.route('/<int:user_id>&<int:user_id2>')
def get_thread_by_user_ids(user_id, user_id2, page_number=1, count=20):
    """
    Gets the thread for a given user_id and user_id2.

    :param user_id: The id of one user
    :param user_id2: The id of the other user

    ******Doesn't return the conversation, just the thread information*****
    """
    thread = Thread.query.filter(or_(
        and_(
            Thread.from_user == user_id,
            Thread.to_user == user_id2
        ),
        and_(
            Thread.from_user == user_id2,
            Thread.to_user == user_id
        )
    )).first()

    return jsonify(thread.serialize())

@message.route('/thread/<int:thread_id>')
@message.route('/thread/<int:thread_id>&p=<int:page_number>')
@message.route('/thread/<int:thread_id>&p=<int:page_number>&c=<int:count>')
def get_thread(thread_id, page_number=1, count=20):
    """
    Get all the messages in a specified thread.

    :param thread_id: The thread id to get the messages for.
    :return: A json response containing the messages.
    """
    thread = Thread.query.filter_by(id=thread_id).first()
    # messages = Message.query \
    #            .filter_by(thread_id=thread_id) \
    #            .order_by(Message.id) \
    #            .paginate(page_number, count)
    
    messages = Message.query.filter_by(thread_id=thread_id).order_by(desc(Message.id)).paginate(page_number, count).items[::-1]


    messages = [m.serialize() for m in messages]
    first_message = messages[0]

    # This works because the user initiating the message
    # is the one that will be sending the transaction request
    # which is why we know they will always be the from user
    # I think this is ok? might have to add the condition for the opposite later
    transactions = Transaction.query \
                    .join(Transaction.listing) \
                    .filter(or_(
                        and_(
                            Transaction.buyer_id == thread.from_user,
                            Listing.seller_id == thread.to_user,
                            Transaction.updated >= first_message['created']
                        ),
                        and_(
                            Transaction.buyer_id == thread.to_user,
                            Listing.seller_id == thread.from_user,
                            Transaction.updated >= first_message['created']
                        )
                    )) \
                    .all()

    transactions = [t.serialize() for t in transactions]

    # Return the data
    combined_sorted = sorted(messages + transactions, key=lambda k: k['created'], reverse=False)
    # return combined_sorted
    return jsonify(combined_sorted)

@message.route('/send', methods=['POST'])
def send_message():
    """Send a new message"""
    content = request.json

    thread = Thread.query.filter(or_(
        and_(
            Thread.from_user == content['from_user'],
            Thread.to_user == content['to_user']
        ),
        and_(
            Thread.from_user == content['to_user'],
            Thread.to_user == content['from_user']
        )
    )).first()

    # Add new thread if it doesn't already exist
    if not thread:
        thread = Thread(from_user=content['from_user'], to_user=content['to_user'])
        db.session.add(thread)
        db.session.commit()

    message = Message(
        thread_id=thread.id, 
        message=content['message'], 
        read=False, 
        sent_by=content['from_user'], 
        is_request=content['is_request']
    )
    db.session.add(message)
    db.session.commit()

    try:
        user = User.query.filter_by(id=content['to_user']).first()
        send_push_message(user.expo_token, message.message, {'data':content['from_user']})
    except Exception as err:
        print('Unable to send push notification!')
        print(err)
    
    return 'success', 200


