"""
transaction.py

API endpoint for handling transactions.

Created by Stephen Andrews, December 6th, 2018.
Updated 1/11/2019
"""

from flask import Blueprint, current_app, jsonify, render_template, request

from app.extensions import db
from app.models import Listing, User, Status, Transaction
from app.utils import verify_session
from sqlalchemy import and_, or_, desc

transaction = Blueprint('transaction', __name__)

@transaction.before_request
def authorize():
    """Require authorization for all views."""
    if current_app.config['DEBUG']:
      return

    if not request.headers.get('authorization'):
        return 'unauthorized', 401
    temp = request.headers['authorization'].split(' ')
    if not len(temp) == 2:
        return 'invalid authorization header'
    if not verify_session(temp[1]):
        return 'unauthorized', 401

@transaction.route('/', methods=['POST'])
def index():
  """Main endpoint used to create or update transactions."""
  payload = request.json
  if not payload:
    return 'Missing JSON payload', 412

  # Update existing transaction
  transaction_id = payload.get('transaction_id', None)
  if transaction_id:
    status = Status.query.filter_by(status=payload['status']).first()
    transaction = Transaction.query.filter_by(id=transaction_id).first()
    transaction.status = status.id
    
    if transaction.status == Status.query.filter_by(status='COMPLETED').first().id:
      listing = Listing.query.filter_by(id=payload['listing_id']).first()
      listing.visible = False
      db.session.add(listing)

    db.session.add(transaction)
    db.session.commit()

    return 'Updated transaction to {}'.format(payload['status']), 200
  
  return create_transaction(payload)

@transaction.route('/user/<int:user_id>')
def transaction_history(user_id):
  """
  Get the transaction history for a specified user ID.

  :param user_id: The ID of the user to get the transaction history of.
  """
  completed_status_id = Status.query.filter_by(status='COMPLETED').first().id
  query = Transaction \
          .query \
          .join(Transaction.listing) \
          .filter(and_(Listing.seller_id == user_id, Transaction.status == completed_status_id)) \
          .all()

  history = {}
  history['total'] = sum([t.listing.sale[0].price for t in query]) 
  history['items'] = [t.serialize() for t in query]

  return jsonify(history)

def create_transaction(payload):
  """
  Creates a new transaction entry and sets its status to PENDING.

  :param payload: A dictionary containing the JSON payload.
  :return: A JSON representation of the created listing.
  """
  transaction = None
  
  try:  
    transaction = Transaction(**payload)
    transaction.status = Status.query.filter_by(status='PENDING').first().id
  except Exception as ex:
    print(ex)
  
  if not transaction:
    return 'Unable to unpack payload to transaction model', 412
  
  db.session.add(transaction)
  db.session.commit()

  return jsonify(transaction.serialize())


