"""
user.py

API endpoints for interacting with a user.

Created by Stephen Andrews, September 11th, 2018.
"""

import hashlib
import uuid

from flask import Blueprint, jsonify, render_template, request

from app.extensions import db
from app.models import BugReport, Listing, PasswordReset, User, VerificationList
from app.utils import send_email, verify_session

PASSWORD_RESET_EMAIL = """Hey {name},

A password reset has been requested for this account. To reset your password please click the link: {link}.

If you did not request a password reset you can ignore this email."""

user = Blueprint('user', __name__)

@user.route('/', methods=['GET'])
@user.route('/<string:username>', methods=['GET'])
def get(username=None):
    """
    Returns a (todo paginated) list of all users. If a username is specified,
    it will return the single user.

    :param username: An option username matching a single user.
    :return: A serialized user object.
    """

    if username:
        user = User.query.filter_by(username=username).first()
        if not user:
            return 'User {} doesn\'t exist'.format(username)

        return jsonify(user.serialize())

    return jsonify([user.serialize() for user in User.query.all()])

@user.route('/register', methods=['POST'])
def register():
    """
    Registers a new user.
    """
    content = request.json
    content['password'] = encrypt_password(content['password'])
    temp = uuid.uuid4()

    # maintain that it is unique
    while(session_id_taken(temp)):
        temp = uuid.uuid4()

    content['session_id'] = temp

    user = User(**content)
    db.session.add(user)
    db.session.commit()

    veri_content = {}
    veri_content['session_id'] = temp
    veri_content['valid'] = True

    verification = VerificationList(**veri_content)
    db.session.add(verification)
    db.session.commit()

    return jsonify(user.serialize())

# Check if a given uuid4 is already taken by another user
def session_id_taken(session_id):
    user = User.query.filter_by(session_id='{}'.format(session_id)).first()
    if not user:
        return False
    return True

@user.route('/login', methods=['POST'])
def login():
    """
    Attempt to login a user.
    """
    data = request.json
    if not data:
        return 'No content received', 400

    user = User.query.filter_by(username=data['username']).first()
    if not user:
        return 'Incorrect login', 401

    # Check password
    if user.password == encrypt_password(data['password']):
        return jsonify(user.serialize()), 200

    return 'Incorrect login', 401

@user.route('/picture', methods=['POST'])
def update_profile_picture():
    """
    Attempt to change a user's profile picture.
    """
    # Require authorization for updating profile picture.
    if not request.headers.get('authorization'):
        return 'unauthorized', 401
    temp = request.headers['authorization'].split(' ')
    if not len(temp) == 2:
        return 'invalid authorization header'
    if not verify_session(temp[1]):
        return 'unauthorized', 401
    data = request.json
    if not data:
        return 'No content received', 400

    user = User.query.filter_by(username=data['username']).first()
    if not user:
        return 'Incorrect username', 401

    picture = data['picture']
    user.profile_picture = picture
    try:
        db.session.add(user)
        db.session.commit()
    except Exception as e:
        print(e)
        return 'Unable to save user!', 500
    return 'Updated profile picture', 200

@user.route('/forgot/<string:email>')
def request_reset(email=None):
    """
    Request a password reset.

    :param email: The email registered for the user requesting the reset.
    """
    if not email:
        return 'No email supplied', 400

    user = User.query.filter_by(email=email).first()
    if user:
        key = str(uuid.uuid4())
        reset_link = 'https://bookstand.io/user/reset/' + key
        msg = PASSWORD_RESET_EMAIL.format(name=user.username, link=reset_link)
        password_reset = PasswordReset(email=user.email, key=key)

        try:
            send_email(user.email, 'Password Reset', msg)
            db.session.add(password_reset)
            db.session.commit()
        except Exception as e:
            print(e)
            return 'Failed to persist password reset entry!', 500

    return 'An email has been sent to {}'.format(email)

@user.route('/reset/', methods=['POST'])
@user.route('/reset/<string:key>', methods=['GET'])
def reset_password(key=None):
    """
    Resets a user's password.

    :param key: The unique key emailed to the user.
    """
    if request.method == 'GET':
        if not key:
            return render_template("errors/404.html"), 404

        reset = PasswordReset.query.filter_by(key=key).first()
        if not reset:
            return render_template("errors/404.html"), 404

        return render_template('home/password_reset.html', key=key)

    reset = PasswordReset.query.filter_by(key=request.form['key']).first()
    user = User.query.filter_by(email=reset.email).first()

    user.password = encrypt_password(request.form['password'])
    try:
        db.session.add(user)
        db.session.commit()
    except Exception as e:
        print(e)
        return 'Unable to save user!', 500

    return 'Password reset!'

@user.route('/changepw/', methods=['POST'])
def change_password():
    """
    Changes a user's password.
    """
    data = request.json
    if not data:
        return 'No content received', 400

    old_pw = data['oldpw']
    new_pw = data['newpw']
    user = User.query.filter_by(username=data['username']).first()
    if not user:
        return 'Incorrect login', 401

    # Check password
    if user.password == encrypt_password(old_pw):
        user.password = encrypt_password(new_pw)
        try:
            db.session.add(user)
            db.session.commit()
        except Exception as e:
            print(e)
            return 'Unable to save user!', 500
    else:
        return 'Old password incorrect', 401       


    return 'Password changed!'

def encrypt_password(password):
    """
    Encrypts a password using SHA256.

    :param password: The password to encrypt.
    :return: The encrypted password.
    """
    m = hashlib.sha256()
    m.update(password.encode('UTF-8'))

    return m.digest()

@user.route('/bugreport/', methods=['POST'])
def bug_report():
    """Submits a user reported bug."""
    data = request.json
    if not data:
        return 'No content received', 400
    
    user = User.query.filter_by(username=data['username']).first()
    if not user:
        return 'User {} not found'.format(data['username']), 401
    
    bug_report = BugReport(user_id=user.id, report=data['report'])
    db.session.add(bug_report)
    db.session.commit()

    return 'Report submitted', 200

@user.route('/listings/<string:username>')
@user.route('/listings/<string:username>/&p=<int:page_number>')
@user.route('/listings/<string:username>/&p=<int:page_number>&c=<int:count>')
def get_user_listings(username=None, page_number=1, count=25):
    """
    Returns all the listings for the specified user that are currently visible.
    
    :param username: The user to get the listings of.
    """
    if not username:
        return '[get_user_listings] No username specified!'
    
    user = User.query.filter_by(username=username).first()
    if not user:
        return '[get_user_listings] User {} not found'.format(username), 401
    
    listings = Listing.query.filter_by(
        seller_id=user.id,
        visible=True
    ).paginate(page_number, count).items

    return jsonify([l.serialize() for l in listings])
