import React, {
  Component
} from 'react';
import {
  FlatList,
  StyleSheet,
  Text,
  View
} from 'react-native';

import API from '../API';
import BookEntryItem from '../views/browse/BookEntryItem';


/**
 * Provides autocomplete functionality when searching for a book via 
 * the Google Books API. The component uses the {@link BookEntryItem}
 * component to return a list of autocomplete suggestions.
 * 
 * Pass the desired search term to this component as `searchTerm` and `onSelect` as
 * a callback function containing the book object returned by the API.
 * ```
 * <Search serachTerm="The Bible" onSelect={someCallbackFunction} />
 * ```
 * 
 * @author John Bieber, Stephen Andrews
 * @since 10/25/18
 */
export default class Search extends Component {
  
  constructor() {
    super();

    this.state = {
      'searchTerm': "",
      'waiting': false
    }
  }

  /**
   * Update search results when a new search term is passed.
   */
  componentDidUpdate() {
    if (this.props.searchTerm.length > this.state.searchTerm.length) {
      this.search(this.props.searchTerm);
    }
  }

  search = (title) => {
    this.setState({searchTerm: title});

    if (this.state.waiting) {
      this.setState({
        waiting: true,
        searchNext: title,
        listings: this.state.listings
      });
      
      return;
    }
    
    this.setState({
      waiting: true,
      listings: this.state.listings
    })

    let cb = (response) => {
      if (response.status != 200) {
        return;
      }

      return response.json().then((data) => {
        let tmp = this.state.searchNext;
        this.setState({
          listings: data,
          waiting: false,
          searchNext: null
        });

        if(tmp){
          this.search(tmp);
        }
      }).catch((error) => {
        console.log("[Search] " + error);
        console.log(response);
      });
    }
    
    API.globalSearchTitle(title, cb);
  }

  wrap = (item) => {
    return {
      book: item
    }
  }

  /**
   * Behavior when an item is selected from the search results.
   * @param isbn The ISBN of the selected book.
   */
  select = (isbn=undefined) => {
    if (!isbn) {
      console.log("[Search] Failed to get ISBN from selected search result!");
      return;
    }

    let cb = (response) => {
      return response.json().then((data) => {
        { this.props.onSelect(data) }
      }).catch((error) => {
        console.log(error);
      });
    }

    API.getBookIDFromISBN(isbn, cb);
  }

  render() {
    if (!this.state.listings) {
      return (
        <View style={[styles.container, {paddingTop: 50}]}>
          <Text style={styles.label}>
            No results found for {this.state.searchTerm}!
          </Text>
        </View>
      );
    }

    return (
      <View style={styles.container}>
        <FlatList
          data={this.state.listings}
          keyExtractor={(item) => item.isbn_13}
          renderItem={({item}) => {
            return <BookEntryItem 
                      press={(isbn_13) => this.select(isbn_13)} 
                      item={this.wrap(item)} 
                      {...this.props} />
            }
          }
        />
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    backgroundColor: '#FDFDFE',
    flex: 1
  },
  label: {
    color: '#464A53',
    fontWeight: '500',
    fontSize: 16,
    margin: 15,
    textAlign: 'center'
  }
});
