import React, {
  Component
} from 'react';
import {
  Platform,
  Image,
  View,
  Text,
  TextInput,
  TouchableOpacity,
  StyleSheet,
  FlatList,
  ActivityIndicator
} from 'react-native';

import Icon from 'react-native-vector-icons/Feather';

import API from '../../API';

import BookEntryItem from './BookEntryItem';
import HorizontalRule from '../../components/HorizontalRule';

/**
 * A {@link FlatList} component used to display available books.
 * @author Ted Bieber, Stephen Andrews
 */
export default class BrowseListView extends Component {
  constructor() {
    super();

    this.state = {
      isLoading: true,
      listings: [],
      search: "",
      updated: false,
      pagesLoaded:0,
    };
  }

  componentDidMount() {
    
    let cb = (response) => {
      books = [];
      return response.json().then((data) => {
        this.setState({
          isLoading: false,
          listings: data,
          pagesLoaded: 1,
          updated: false
        });
      }).catch((error) => {
        console.log(error);
      });
    }

    API.getListedBooks(cb);
    return;
  }

  

  /**
   * Show search results if length of search is > 0.
   */
  componentWillUpdate = () => {
    if (this.state.updated) {
      if (this.state.search.length > 0) {
        this.loadListings('title=' + this.state.search);
      } else {
        this.loadListings();
      }
      this.setState({updated: false});
    }
  }

  loadListings = (params=null) => {
    this.state.listings = [];

    let cb = (response) => {
      return response.json().then((data) => {
          this.setState({
            isLoading: false,
            listings: data,
            pagesLoaded: 1,
            updated:false,
          });
        }).catch((error) => {
          console.log(error);
        });
    }

    if (params != null) {
      API.getListedBooks(cb, params);
    } else {
      API.getListedBooks(cb);
    }
  }

  loadNextPage = () => {
    let cb = (response) => {
      response.json().then((data)=> {
        this.setState({
          isLoading: false,
          listings: this.state.listings.concat(data),
          pagesLoaded: this.state.pagesLoaded+1,
          updated:false
        });
      }, ()=>{
      }).catch((error)=>{
        console.log(error);
      });
    }
    let params = null;
    if (this.state.search.length > 0) {
      params = 'title=' + this.state.search;
    } 
    API.getListedBooks(cb, params, this.state.pagesLoaded+1);
  }

  refreshPage = () => {
    let cb = (response) => {
      response.json().then((data)=> {
        this.setState({
          isLoading: false,
          listings: data,
          pagesLoaded: 1,
          updated:true
        });
      }).catch((error)=>{
        console.log(error);
      });
    }
    let params = null;
    if (this.state.search.length > 0) {
      params = 'title=' + this.state.search;
    } 
    API.getListedBooks(cb, params);
  }

  render() {
    {/* Show loading indicator until API returns response */}
    if (this.state.isLoading) {
      return (
        <View style={{flex: 1, justifyContent: 'center'}}>
          <ActivityIndicator />
        </View>
      );
    }

    if (this.props.navigation.state && this.props.navigation.state.params && this.props.navigation.state.params.refresh){
      this.props.navigation.state.params.refresh = false;
      this.loadListings();
    }

    return (
      <View style={styles.container}>
        <View 
          flexDirection="row"
          justifyContent="space-between"
          alignItems="center"
          style={{marginHorizontal: 25, backgroundColor: '#FFF'}}>

          <TextInput 
            onChangeText={(text) => this.setState({updated: true, search: text})}
            placeholder="Search for a title..."
            style={styles.input} />
          <Icon name="search" size={25} />
          
        </View>
        <HorizontalRule style={{ margin: 20}} />

        <FlatList
          data={this.state.listings}
          keyExtractor={(item) => item.book.isbn_13}
          renderItem={({item}) => <BookEntryItem item={item} {...this.props} />}
          onEndReachedThreshold={0.25}
          onEndReached={this.loadNextPage}
          refreshing={false}
          onRefresh={this.refreshPage}
          
        />
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    flex: 1
  },
  heading: {
    color: '#FFFFFF',
    fontSize: 42,
    fontFamily: Platform.OS === 'ios' ? 'Helvetica' : 'Roboto',
    fontWeight: '500',
    marginBottom: 15,
    marginLeft: 15
  },
  input: {
    backgroundColor: '#FFF',
    fontSize: 18,
    fontWeight: '700',
    width: '90%'
  }

});
