import React, {
  Component
} from 'react';

import {
  Platform,
  Image,
  ScrollView,
  StyleSheet,
  Text,
  TouchableOpacity,
  View
} from 'react-native';

import Icon from 'react-native-vector-icons/Feather';

import API from '../../API';
import * as Constants from '../../Constants';

// Import components
import ImageCarousel from '../../components/ImageCarousel';
import BookEntry from '../../components/BookEntry';


/**
 * Shows a detailed view for a single listing 
 * @author Stephen Andrews
 * @since 11.24.18
 */
export default class DetailedView extends Component {

  /**
   * The action to perform when the user presses the buy button.
   * @param listing The listing object.
   * @param user_id The ID of the current user browsing the listing.
   */
  buyAction = (listing, user_id) => {
    // TODO: Ideally to 'go back' screen should be this page, not all messages screen
    let cb = (response) => {
      this.props.navigation.navigate('Conversation', {
        'user_id1': listing.seller.user_id,
        'user_id2': user_id,
      });
    }
    API.sendMessage(user_id, listing.seller.user_id, "Hi, ", () => {});    
    API.sendTransaction(user_id, listing.listing_id, "PENDING", null, cb);
  }

  /**
   * Cancel an active listing.
   * @param listing The active listing.
   */
  cancelAction = (listing) => {
    // TODO: Add alert confirmation
    let cb = (response) => {
      this.props.navigation.goBack();
    }

    API.removeListing(listing.listing_id, cb);
  }

  /**
   * Render the navigation header with a back button.
   */
  renderHeader = () => {
    return (
      <TouchableOpacity
        onPress={() => this.props.navigation.goBack()}
        style={{
          position: 'absolute',
          top: 45,
          left: 15,
          zIndex: 1
        }}>

        <Icon name="arrow-left" color='#FFF' size={30} style={styles.backButton} />
      </TouchableOpacity>
    );
  }

  renderSellerInfo = (listing) => {
    return (
      <View flexDirection="row" marginBottom={15}>
        <Image 
            style={styles.profilePicture}
            source={{uri: listing.seller.profile_picture}} />
        
        <View justifyContent="center">
          <Text>{listing.seller.username}</Text>
          <Icon style={{'color': 'black'}} name="star" size={20} />
        </View>
      </View>
    );
  }

  /**
   * Render a label.
   * @param text The text to display.
   */
  renderLabel = (text) => {
    return (
      <Text
        style={{
          'fontSize': 18,
          'fontWeight': 'bold',
          marginBottom: 5
        }}>
        { text }
    </Text>
    );
  }

  /**
   * Render the details provided by the user such as price, condition,
   * description, and the user who created the listing.
   * @param listing The listing object.
   */
  renderListingDetails = (listing) => {
    return (
      <View style={styles.contentContainer}>
        <View height={130} marginBottom={30}>
          <BookEntry book={listing.book} />
        </View>

        {/* Main content */}
        <View>
          { this.renderLabel('Description') }
          <Text style={styles.description}>{listing.description}</Text>
        </View>

        { this.renderSellerInfo(listing) }
      </View>
    );
  }

  /**
   * If the viewing user is the user who created the listing, a cancel button 
   * will be shown. Otherwise, an option to purchase the book will be shown.
   * @param listing The listing object.
   * @param user_id The ID of the user currently viewing the listing.
   */
  renderFooterContent = (listing, user_id) => {
    if (user_id == listing.seller.user_id) {
      return (
        <View
          alignItems="center"
          flexDirection="row"
          justifyContent="space-between"
          style={{
            backgroundColor: '#FFF',
            padding: 20,
            shadowColor: 'black',
            shadowOffset: {width: 0, height: -5},
            shadowRadius: 5,
            shadowOpacity: 0.10
        }}>
          <View>
            { this.renderLabel('Cancel listing') }
            <Text style={{color: '#CDCFD1', fontSize: 14, fontWeight: '500'}}>Please click to cancel</Text>
          </View>


          <TouchableOpacity 
            onPress={() => this.cancelAction(listing)}
            style={styles.cancelButton}>
            <Text style={styles.buttonText}>Cancel</Text>
          </TouchableOpacity>
        </View>
      );
    } else {
      return (
        <View
          alignItems="center"
          flexDirection="row"
          justifyContent="space-between"
          style={{
            backgroundColor: '#FFF',
            padding: 20,
            shadowColor: 'black',
            shadowOffset: {width: 0, height: -5},
            shadowRadius: 5,
            shadowOpacity: 0.10
        }}>
          <View>
            { this.renderLabel(listing.condition) }
            <Text style={{color: '#CDCFD1', fontSize: 14, fontWeight: '500'}}>Please click to pay</Text>
          </View>
    
    
          <TouchableOpacity 
            onPress={() => this.buyAction(listing, user_id)}
            style={styles.buyButton}>
            <Text style={styles.buttonText}>${listing.price}</Text>
          </TouchableOpacity>
        </View>
      );
    }
  }

  supplementPhotos = (currentPhotos) => {
    let outPhotos = [].concat(currentPhotos);
    for (let i = 0; i + currentPhotos.length < 4; i++) {
      outPhotos.push('https://s3.amazonaws.com/bookstand/photo/puppy/picture' + (1 + Math.floor(Math.random() * Math.floor(96)) + '.jpeg'));
    }

    return outPhotos;
  }

  generateOverlays = (currentPhotos) => {
    let outOverlays = [];
    for (let i = 0; i < currentPhotos.length; i++){
      outOverlays.push('');
    }
    for (let i = currentPhotos.length; i < 4; i++){
      outOverlays.push('Placeholder Puppy');
    }
    return outOverlays;
  }

  render() {
    // Since this is coming from navigation params, it's easiest to get
    // here since the navigation prop is null in other methods such as constructor
    let { listing } = this.props.navigation.state.params;
    let { user_id } = this.props.navigation.state.params;

    return (
      <View style={styles.container}>
        { this.renderHeader() }

        <ScrollView>
          <ImageCarousel 
            photos={this.supplementPhotos(listing.photos)} 
            overlays={this.generateOverlays(listing.photos)} />
          
          { this.renderListingDetails(listing) }
        </ScrollView>

        {/* Anchored at bottom */}
        { this.renderFooterContent(listing, user_id) }
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: '#FDFDFE'
  },
  contentContainer: {
    flex: 1,
    padding: 20
  },
  backButton: {
    shadowColor: 'black',
    shadowOffset: {width: 0, height: 0},
    shadowRadius: 2,
    shadowOpacity: 0.40
  },
  buyButton: {
    backgroundColor: Constants.PRIMARY_BLUE,
    padding: 10,
    borderRadius: 5,
    width: 100,
    shadowColor: Constants.PRIMARY_BLUE,
    shadowOffset: {width: 0, height: 0},
    shadowRadius: 5,
    shadowOpacity: 0.50
  },
  buttonText: {
    color: '#FFF',
    fontSize: 18,
    fontFamily: Platform.OS === 'ios' ? 'Helvetica' : 'Roboto',
    textAlign: 'center'
  },
  cancelButton: {
    backgroundColor: Constants.PRIMARY_RED,
    padding: 10,
    borderRadius: 5,
    width: 100,
    shadowColor: Constants.PRIMARY_RED,
    shadowOffset: {width: 0, height: 0},
    shadowRadius: 5,
    shadowOpacity: 0.50
  },
  description: {
    fontSize: 14,
    paddingVertical: 15,
    paddingBottom : 15
  }, 
  profilePicture: {
    borderRadius: 25,
    marginRight: 15,
    width: 50, 
    height: 50
  }

});
