import React, { Component } from 'react';

import {
  Modal,
  Platform,
  View,
  StyleSheet,
  Text,
  TouchableOpacity
} from 'react-native';

import Icon from 'react-native-vector-icons/Feather';
import MultiSlider from '@ptomasroos/react-native-multi-slider';

import API from "../../API";
import * as Constants from "../../Constants";

/**
 * Contains layout for the filter modal used in the {@link SaleListView} screen.
 * @author Stephen Andrews
 * @since 11.23.18
 */
export default class FilterModal extends Component {

  constructor() {
    super();

    this.state = {
      minSlider: 0,
      maxSlider: 0,

      lowestPrice: false,
      mostRecent: false,
      maxPrice: -1,
      maxSlider: -1
    }
  }

  componentDidUpdate() {
    if (this.state.maxPrice > -1) { 
      return;
    }

    this.setState({
      maxPrice: this.props.maxPrice,
      maxSlider: this.props.maxPrice
    });
  }

  /**
   * Make an API call to the backend applying the selected filters.
   * Invokes the callback from the parent passing the listings returned.
   */
  applyFilter = () => {
    // let cb = response => {
    //   response.json().then(data => {
    //     this.props.onPress();
    //     this.props.callback(data);
    //   });
    // } // old

    var params = []

    // Filter by lowest price
    if (this.state.lowestPrice) {
      params.push('order=price')
    }

    // Filter by newest listings
    if (this.state.mostRecent) {
      params.push('order=new')
    }

    // Price range filtering
    params.push('low=' + this.state.minSlider);
    params.push('high=' + this.state.maxSlider);
  
    // API.getListings(this.props.bookId, cb, params.join('&')) // old
    this.props.callback(params.join('&'));
  }
  
  /**
   * Renders the header section containing the title and close button.
   */
  renderHeader = () => {
    return (
      <View style={styles.header}>
        <Text style={styles.heading}>Filters</Text>

        {/* This needs to be after the text element due to absolute positioning.
            Otherwise, you won't be able to click the button. */}
        <TouchableOpacity 
          onPress={this.props.onPress}
          style={styles.dismissButton}>

          <Icon name="x" color="#FFF" size={30} />
        </TouchableOpacity>
      </View>
    );
  }

  /**
   * Begins the start of a new section with the specified name.
   */
  renderSectionHeader = (name) => {
    return (
      <View flexDirection="row" alignItems="center" marginBottom={25}>
        <Text style={styles.sectionHeading}>{name}</Text>
        <View style={{
          backgroundColor: '#FFF',
          marginLeft: 20,
          flex: 1,
          height: 1,
          opacity: .30
        }} />
      </View>
    );
  }

  /**
   * Renders a check box button with a label.
   * @param name The name of the label.
   * @param state The key used in the state dictionary.
   */
  renderCheckbox = (name, state) => {
    let checked = this.state[state];
    let style = checked ? styles.checkboxButtonActive : styles.checkboxButtonInactive;

    // Need to set state this way since we're accessing state via a string
    let newState = {}
    newState[state] = !checked;

    return (
      <View 
        flexDirection="row"
        justifyContent="space-between"
        marginBottom={15}>

        <Text style={{
          color: '#FFF',
          fontSize: 16,
          fontWeight: '700'
        }}>{name}</Text>

        <TouchableOpacity 
          onPress={() => { this.setState(newState) }}
          style={styles.checkboxButton}>
          
          {/* This is accent in middle of button */}
          <View style={style} />
        </TouchableOpacity>
      </View>
    );
  }

  /**
   * Renders the sort section.
   */
  renderSortSection = () => {
    return (
      <View marginBottom={60}>
        { this.renderSectionHeader('SORT') }

        { this.renderCheckbox('Lowest Price', 'lowestPrice') }
        { this.renderCheckbox('Most Recent', 'mostRecent') }
      </View>
    );
  }

  /**
   * Renders the price filter section.
   */
  renderPriceSection = () => {
    return (
      <View marginBottom={60}>
        { this.renderSectionHeader('PRICE') }
        { this.renderPriceFilter() }
      </View>
    );
  }

  /**
   * Renders a custom slider marker.
   */
  customMarker = (slider) => {
    return (
      <View style={{alignItems: 'center'}}>
        <Text style={styles.sliderLabel}>${slider.currentValue}</Text>
        
        <View style={styles.marker}>
            <View 
              style={{
                backgroundColor: Constants.PRIMARY_BLUE,
                borderRadius: 7.5,
                height: 15,
                width: 15
              }} />
          </View>
      </View>
    );
  }

  /**
   * Renders the price range slider.
   */
  renderPriceFilter = () => {
    let onValuesChange = values => {
      this.setState({
        minSlider: values[0],
        maxSlider: values[1]
      });
    }

    return (
      <View alignItems="center">
        <MultiSlider
          onValuesChange={onValuesChange}
          values={[this.state.minSlider, this.state.maxSlider]} 
          min={this.state.minPrice}
          max={this.state.maxPrice} 
          customMarker={this.customMarker} 
          selectedStyle={{
            backgroundColor: '#FFF'
          }}
          unselectedStyle={{
            backgroundColor: '#FFF',
            opacity: .3
          }}
          containerStyle={{
            justifyContent: 'flex-end'
          }}
          trackStyle={{
            marginTop: 25,
          }} />
      </View>
    );
  }

  render() {
    return (
      <Modal
        animationType="slide"
        transparent={true}
        visible={this.props.visible}>

        <View style={styles.container}>
          { this.renderHeader() }

          <View style={styles.contentContainer}>
            { this.renderSortSection() }
            { this.renderPriceSection() }
          </View>

          <TouchableOpacity 
            onPress={this.applyFilter}
            style={styles.applyButton}>
            <Text style={styles.applyButtonText}>APPLY</Text>
          </TouchableOpacity>
        </View>
      </Modal>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: Constants.PRIMARY_BLUE
  },
  contentContainer: {
    flex: 1,
    margin: 30
  },
  applyButton: {
    backgroundColor: '#FFF',
    height: 75,
    padding: 25
  },
  applyButtonText: {
    color: Constants.PRIMARY_BLUE,
    fontSize: 14,
    fontWeight: '700',
    textAlign: 'center'
  },
  checkboxButton: {
    backgroundColor: '#FFF',
    borderRadius: 4,
    justifyContent: 'center',
    alignItems: 'center',
    height: 25,
    width: 25,
    opacity: 0.4
  },
  checkboxButtonInactive: {
    backgroundColor: Constants.PRIMARY_BLUE,
    borderRadius: 2,
    width: 20,
    height: 20
  },
  checkboxButtonActive: {
    backgroundColor: '#FFF',
    borderRadius: 2,
    width: 20,
    height: 20
  },
  dismissButton: {
    position: 'absolute',
    left: 20
  },
  header: {
    height: 100,
    justifyContent: 'flex-end',
  },
  heading: {
    color: '#FFF',
    fontSize: 24,
    fontFamily: Platform.OS === 'ios' ? 'Helvetica' : 'Roboto',
    fontWeight: 'bold',
    textAlign: 'center'
  },
  marker: {
    alignItems: 'center',
    justifyContent: 'center',
    backgroundColor: '#FFF',
    borderRadius: 15,
    height: 30,
    width: 30,
    shadowColor: 'black',
    shadowOffset: {width: 0, height: 3},
    shadowRadius: 2,
    shadowOpacity: 0.50
  },
  sectionHeading: {
    color: '#FFFFFF',
    fontSize: 14,
    fontWeight: '700',
    opacity: .70
  },
  sliderLabel: {
    color: '#FFF',
    fontSize: 18,
    fontFamily: Platform.OS === 'ios' ? 'Helvetica' : 'Roboto',
    fontWeight: 'bold',
    marginBottom: 30,
    textAlign: 'center'
  }
});
