import React, { Component } from "react";
import {
  ActivityIndicator,
  Platform,
  FlatList,
  StyleSheet,
  Text,
  TouchableOpacity,
  View,
} from "react-native";

import API from "../../API";
import FilterModal from "./FilterModal";
import SaleItem from "./SaleItem";

/**
 * List view containing all of the postings for a specific book.
 * @author Stephen Andrews
 * @since 10.4.2018
 */
export default class SaleListView extends Component {
  constructor() {
    super();

    this.state = {
      isLoading: true,
      listings: [],
      filterVisible: false,
      maxPrice: 0,
      pagesLoaded: 0,
      filters:null,
    };
  }

  componentWillMount() {
    let cb = response => {
      return response.json().then(data => {
        this.setState({
          isLoading: false,
          listings: data,
          pagesLoaded: 1,
        });

        this.setState({maxPrice: this.getMaxListingPrice()});
      });
    };
    API.getListings(this.props.bookId, cb);
  }

    /**
   * Renders a the filter button using absolute position.
   */
  renderFilterButton = () => {
    return (
      <View style={{position: 'absolute', alignItems: 'center', top: '92%', bottom: 0, left: 0, right: 0, zIndex: 1}}>
        <TouchableOpacity 
          onPress={() => { this.setState({'filterVisible': true})}}
          style={styles.filterButton}>

          <Text style={styles.filterText}>Filter</Text>
        </TouchableOpacity>
      </View>
    );
  }

  /**
   * Dismiss filter modal.
   */
  dismiss = () => {
    this.setState({'filterVisible': false});
  }

  /**
   * The callback function for filtering.
   * @param data The listing results returned from the API after being filtered.
   */
  fitlerCallback = (fitler) => { // TODO: fitler?
    // this.setState({listings: data}); // OLD
    this.state.filters = fitler;

    let cb = response => {
        response.json().then(data => {
          this.setState({
            listings: data,
            filterVisible: false,
            pagesLoaded: 1,
          });
          this.state.maxPrice = this.getMaxListingPrice();
        }, () => {
        });
      }
    API.getListings(this.props.bookId, cb, this.state.filters);
  }

  /**
   * Gets the price of the most expensive listing to be passed
   * to the filter modal for the price range slider.
   * @return The most expensive listing price.
   */
  getMaxListingPrice = () => {
    var max = 0;
    for (var listing of this.state.listings) {
      if (listing.price > max) {
        max = listing.price;
      }
    }

    return max;
  }

  loadNextPage = () => {

    let cb = (response) => {
      response.json().then((data) => {
        this.setState({
          isLoading: false,
          listings: this.state.listings.concat(data),
          pagesLoaded: this.state.pagesLoaded+1,
          maxPrice: this.getMaxListingPrice()
        });
      }, () => {
      });
    }
    // API Call, ask for pages loaded+1
    API.getListings(this.props.bookId, cb, this.state.filters, this.state.pagesLoaded+1);
    
  }

  refreshPage = () => {
    let cb = response => {
      return response.json().then(data => {
        this.setState({
          isLoading: false,
          listings: data,
          pagesLoaded: 1,
          maxPrice: this.getMaxListingPrice()
        });

      });
    };

    API.getListings(this.props.bookId, cb, this.state.filters);
  }

  render() {
    {/* Show loading indicator until API returns response */}
    if (this.state.isLoading) {
      return (
        <View style={{ flex: 1, justifyContent: "center" }}>
          <ActivityIndicator />
        </View>
      );
    }

    return (
      <View style={styles.container}>
        <FilterModal 
          callback={this.fitlerCallback}
          maxPrice={this.state.maxPrice}
          onPress={this.dismiss}
          visible={this.state.filterVisible} 
          {...this.props} />

        { this.renderFilterButton() }

        <FlatList
          data={this.state.listings}
          keyExtractor={item => item.listing_id + ""}
          renderItem={({ item }) => <SaleItem listing={item} {...this.props}/>}
          onEndReachedThreshold={0.25}
          onEndReached={this.loadNextPage}
          refreshing={false}
          onRefresh={this.refreshPage}
          removeClippedSubviews={true}
        />
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    flex: 1
  },
  heading: {
    color: "#FFFFFF",
    fontSize: 42,
    fontFamily: Platform.OS === "ios" ? "Helvetica" : "Roboto",
    fontWeight: "500",
    marginBottom: 15,
    marginLeft: 15
  },
  filterButton: {
    backgroundColor: '#000',
    borderRadius: 15,
    paddingVertical: 5,
    paddingHorizontal: 15,
    position: 'absolute',
    shadowColor: 'black',
    shadowOffset: {width: 0, height: 0},
    shadowRadius: 5,
    shadowOpacity: 0.70,  
  },
  filterText: {
    color: '#FFF',
    fontFamily: Platform.OS === 'ios' ? 'Helvetica' : 'Roboto', 
    fontSize: 14,
  }
});
