import React, { Component } from 'react';
import { 
  Image,
  Platform,
  StyleSheet,
  Text, 
  View, 
  TouchableOpacity } from 'react-native';

import { Camera, Permissions, ImageManipulator } from 'expo';
import Icon from 'react-native-vector-icons/Feather';

import SplitView from '../../components/SplitView';


/**
 * A camera view for taking pictures with the phone's camera.
 * @author Stephen Andrews, John Bieber
 * @since 11.2.2018
 */
export default class CameraView extends Component {

  constructor() {
    super(); 

    this.state = {
      'currentPhoto': null,
      'hasCameraPermission': null,
      'isTakingPhoto': false,
      'type': Camera.Constants.Type.back,
    };
  }
  
  async componentDidMount() {
    const { status } = await Permissions.askAsync(Permissions.CAMERA);
    this.setState({ hasCameraPermission: status === 'granted' });
  }

  /**
   * Takes a photo and updates the `currentPhoto` state with the
   * captured image.
   */
  capture = async () => {
    if (this.camera) {
      let isIphone = Platform.OS === 'ios'

      // This timeout is a workaround for a bug in the Expo camera module
      // See: https://github.com/expo/expo/issues/2288#issuecomment-458808770
      setTimeout(() => { this.setState({isTakingPhoto: true})}, 1);
      let photo = await this.camera.takePictureAsync({
        // 'quality': isIphone ? 0.1 : 0.25,
        'skipProcessing': true
      });
      this.setState({ isTakingImage: false });

      let correctHeight = isIphone ? 1000*(photo['height']/photo['width']) : 1000*(photo['width']/photo['height']);
      let newPhoto = await ImageManipulator.manipulateAsync(photo.uri, [{resize: {width: 1000, height: correctHeight}}], {format: 'png', compress: isIphone ? 0.1 : 0.25});
      this.setState({currentPhoto: newPhoto});
    }
  };

  confirm = () => {
    this.props.navigation.navigate(
      this.props.navigation.state.params.returnScreen, 
      {photo: this.state.currentPhoto}
    );
  }

  /**
   * Render the camera viewfinder.
   */
  renderCamera = () => {
    return (
      <Camera 
        autoFocus="on"
        ref={ref => { this.camera = ref; }}
        type={this.state.type} 
        style={{ flex: 1 }} />
    );
  }

  /**
   * Renders the view finder with either:
   *  1. An activate camera view
   *  2. The photo taken by the user
   */
  renderViewfinder = () => {
    if (!this.state.currentPhoto) {
      return this.renderCamera();
    }

    let photo = this.state.currentPhoto;
    return <Image flex={1} source={{uri: photo.uri}} />;
  }

  /**
   * Renders a button to clear the current image so the 
   * user can retake the photo. Only rendered if the current
   * photo is not null.
   */
  renderRetakeButton = () => {
    return (
      <TouchableOpacity
        onPress={() => this.setState({currentPhoto: null})} 
        style={{
          padding: 15
        }}>
        
        <Text style={{
          fontSize: 18,
          fontWeight: '600',
          color: '#CDCFD1'
        }}>Retake</Text>
      </TouchableOpacity>
    );
  }

  /**
   * Renders the retake and capture buttons.
   */
  renderControls = () => {
    let leftComponent = this.state.currentPhoto != null ? this.renderRetakeButton() : <View />

    return (
      <SplitView 
        left={leftComponent} 
        center={this.renderCaptureButton()} 
        style={{marginHorizontal: 30}}/>
    )
  }

  /**
   * Renders the capture button which is used to:
   *  1. Take a photo
   *  2. Confirm a photo
   */
  renderCaptureButton = () => {
    let hasPhoto = this.state.currentPhoto != null;

    return (
      <View 
        alignItems="center"
        justifyContent="center"
        style={styles.captureContainer}>

        <TouchableOpacity
          onPress={() => hasPhoto  ? this.confirm() : this.capture() }
          style={styles.captureButton}>

          {hasPhoto ? <Icon color="#FFF" name="thumbs-up" size={35} /> : <Text></Text> }
        </TouchableOpacity>
      </View>
    );
  }

  render() {
    const { hasCameraPermission } = this.state;
    if (hasCameraPermission === null) {
      return <View />;
    } else if (hasCameraPermission === false) {
      return <Text>No access to camera</Text>;
    } else {
      return (
        <View style={{ flex: 1 }}>
          { this.renderViewfinder() }

          <View style={styles.bottomMenu}>
            { this.renderControls() }
            {/* { this.renderCaptureButton() } */}
          </View>
        </View>
      );
    }
  }
}

const styles = StyleSheet.create({
  container: {
    backgroundColor: '#F6F6FB',
    flex: 1,
    overflow: 'hidden',
    paddingTop: 5
  },
  bottomMenu: {
    backgroundColor: '#FFF',
    flex: 0.50,
    flexDirection: 'row',
    justifyContent: 'center',
  },
  captureContainer: {
    backgroundColor: '#FFF',
    borderRadius: 100,
    borderColor: '#61A3F5',
    borderWidth: 3,
    width: 100,
    height: 100,
  },
  captureButton: {
    alignItems: 'center',
    justifyContent: 'center',
    backgroundColor: '#61A3F5',
    borderRadius: 150,
    width: 88,
    height: 88
  },
});
