import React, {
  Component
} from 'react';
import {
  Alert,
  Image,
  View,
  Text,
  TextInput,
  TouchableOpacity,
  StyleSheet,
} from 'react-native';

import Icon from 'react-native-vector-icons/Feather';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { NavigationActions } from 'react-navigation';

import API from '../../API';
import * as Language from '../../Language';
import Search from '../../components/Search';
import StorageManager from '../../StorageManager';

/**
 * Allows user's to create new listings on the app.
 * @author John Bieber, Stephen Andrews
 * @since x/x/x
 */
export default class CreateListingForm extends Component {
  
  constructor() {
    super();

    this.state = {
      'userID': "",
      'book': null,
      'bookSelected': false,
      'condition': "",
      'description': "",
      'price': "",
      'title': "",
      'photos': []
    };
  }

  componentDidMount() {
    StorageManager.getItem('user_id', (value) => {
      this.setState({userID: value});
    })
  }

  /**
   * Check to see if the user scanned a book with the barcode scanner.
   * If scanned, the state is updated with the data for that book.
   */
  componentDidUpdate() {
    // Determines if a child component returned data via navigation
    let hasReceivedData = this.props.navigation.state.params != null;
    if (!hasReceivedData) {
      return;
    }

    // User scanned book
    let hasScanned = this.props.navigation.state.params.bookInfo != null;
    if (!this.state.book && hasScanned) {
      this.setState({
        book: this.props.navigation.state.params.bookInfo
      });
    }

    // User photos
    let photoTaken = this.props.navigation.state.params.photo;
    if (photoTaken) {
      currentPhotos = this.state.photos;
      currentPhotos.push(photoTaken);
      this.props.navigation.state.params.photo = null;
      this.setState({
        'photos': currentPhotos
      });
    }

    // User selected condition
    let conditionSelected = this.props.navigation.state.params.condition;
    let newCondition = this.state.condition != conditionSelected;
    if (conditionSelected != null && newCondition) {
      this.setState({
        'condition': conditionSelected
      });
    }
  }

  /**
   * Validates the form and sends an API request to create a listing.
   */
  attemptCreateListing = () => {
    if (!this.state.book) {
      Alert.alert("Please select a book");
      return;
    }
    if (this.state.userID === "") {
      Alert.alert("Please enter your User ID");
      return;
    }
    if (this.state.condition === "") {
      Alert.alert("Please enter the condition of your book");
      return;
    }
    if (this.state.description === "") {
      Alert.alert("Please enter a description for your sale");
      return;
    }
    if (this.state.price === "") {
      Alert.alert("Please enter a price for your book.");
      return;
    }
    
    this.confirmListing();
  }

  /**
   * Sends an {@link Alert} to the user requesting confirmation to create the listing. If the user
   * confirms, the listing will go public.
   */
  confirmListing = () => {
    let createListing = () => {
      StorageManager.getItem('username', (username) => {
        API.uploadPhotos(
          this.state.photos,
          username,
          true,
          (response) => {
            if (response.status == 200) {
            }
          },
          (urls) => {
            API.createSale(
              this.state.userID,
              this.state.book.id,
              this.state.condition, 
              this.state.description, 
              this.state.price,
              urls,
              (response) => {
                if (response.status == 200) {
                  this.props.navigation.navigate('Listings');
                    this.resetState();
                }
              }
            );
          }
        );
      }, () => {
        console.log("Failed to load username from storage manager.");
      });
    }

    Alert.alert(
      "Confirm",
      Language.CONFIRM_LISTING_MSG,
      [
        {text: 'Cancel', style: 'cancel'},
        {text: 'Confirm', onPress: () => createListing()}
      ]
    );
  }

  /**
   * Resets the state of the component to prevent old data from showing.
   */
  resetState = () => {
    this.props.navigation.setParams({condition: "", bookInfo:null});

    this.setState({
      book: null,
      bookSelected: false,
      description: "",
      price: "",
      title: "",
      photos: [],
    });
  }

  /**
   * Renders the search component allowing a user to search for a book using autocomplete
   * suggestions or scan a barcode.
   */
  renderSearchField = () => {
    var title = this.state.book != null ? this.state.book.title : this.state.title;

    return (
      <View 
        flexDirection="row"
        justifyContent="space-between"
        style={{backgroundColor: '#FFF'}}>

        <TextInput
          autoCapitalize="none"
          autoCorrect={false}
          clearButtonMode="always"
          flexGrow={1}
          maxLength={100} 
          onChangeText={(text) => this.setState({title: text}) }
          placeholder="Book title e.g. The Bible"
          returnKeyType="done"
          style={styles.input}
          text={title}
          value={title}
          />
        
        {/* Wrap icon in a view to center veritcally within the container */}
        <View justifyContent="center">
          <Icon.Button 
            backgroundColor="transparent"
            color="#4A4A4A"
            name="camera"
            onPress={() => this.props.navigation.navigate('Barcode', {returnScreen: 'CreateListing'})}
            size={35} 
            />
        </View>
      </View>
    );
  }

  /**
   * Renders a blank space to separate content based on the specified height.
   * @param label An optional label to render in the separator.
   */
  renderSeparator = (label="") => {
    if (label) {
      return (
        <View textAlign="center">
          <Text style={styles.label}>{label}</Text>
        </View>
      ); 
    } else {
      return (
        <View minHeight={30} />
      );
    }
  }

  /**
   * Renders a {@link TouchableOpacity} component for adding images
   * to the current listing. If there are already four images attached
   * to the listing, the button will not be shown and an empty view will
   * be rendered instead.
   * @return A {@link TouchableOpacity} for adding images to the listing.
   */
  renderAddPhotoButton = () => {
    if (this.state.photos.length >= 4) {
      return <View></View>
    }

    return (
      <TouchableOpacity 
        onPress={() => this.props.navigation.navigate('CameraView', {returnScreen: 'CreateListing'})}
        style={styles.thumbnailButton}>

        <Icon 
          color="#4A4A4A"
          name="plus" 
          size={35}/>
      </TouchableOpacity>
    );
  }

  /**
   * Returns a list of {@link Image} components containing the photos
   * the user has taken of their listing.
   * @returns A list of {@link Image} components.
   */
  renderPhotos = () => {
    if (!this.state.photos.length > 0) {
      return <View></View>
    }

    photos = [];
    for (var photo of this.state.photos) {
      photos.push(
        <Image 
          source={{uri: photo.uri}}
          style={{
            'borderRadius': 4,
            'marginRight': 10,
            'width': 75,
            'height': 75
          }} />
      );
    }

    return photos;
  }

  /**
   * Renders the photo upload section for users to attach images to their post.
   */
  renderPhotoUpload = () => {
    return (
      <View
        backgroundColor="#FFF"
        minHeight={50}
        padding={15}>
        
        <View
          flexDirection="row"
          justifyContent="space-between"
          paddingBottom={5}>
          
          <Text style={{fontSize: 14, fontWeight: '500'}}>Photos</Text>
          <Text style={{color: '#C7C7CC', fontSize: 14, fontWeight: '500'}}>Max 4</Text>
        </View>

        <View flexDirection="row">
          { this.renderPhotos() }
          { this.renderAddPhotoButton() }
        </View>
      </View>
    );
  }

  /**
   * Renders a selection descriptor that will navigate to another screen when clicked.
   * @param name The name of the descriptor.
   * @param toScreen The screen to navigate to on press.
   */
  renderSelectDescriptor = (name, toScreen) => {
    let key = name.toLowerCase(); // Corresponds to state dictionary
    let text = this.state[key].length > 0 ? this.state[key] : "Select";

    return (
      <TouchableOpacity 
        onPress={() => this.props.navigation.navigate(toScreen, {returnScreen: 'CreateListing'})}
        style={styles.descriptorButton}>
        
        {/* Position label and action labels on opposite edges of screen */}
        <View
          flexDirection="row"
          justifyContent="space-between">

          <Text style={{fontSize: 14, fontWeight: '500'}}>{name}</Text>

          <View flexDirection="row">
            <Text style={{color: '#C7C7CC', fontSize: 14, fontWeight: '500'}}>{text}</Text>
            <Icon style={{marginTop: -2}} name="chevron-right" color="#C7C7CC" size={20} />
          </View>
        </View>
      </TouchableOpacity>
    );
  }

  /**
   * Renders a text input descriptor for the specified name.
   * @param name The name of the descriptor.
   */
  renderPriceDescriptor = (name, keyboardType="default") => {
    return (
      <TouchableOpacity style={styles.descriptorButton} onPress={() => this.priceInput.focus()}>
        
        {/* Position label and text input on opposite edges of screen */}
        <View
          flexDirection="row"
          justifyContent="space-between">

          <Text style={{fontSize: 14, fontWeight: '500'}}>{name}</Text>

          <TextInput
            keyboardType={keyboardType}
            onChangeText={(text) => this.setState({price: text})}
            // onSubmitEditing={() => this.descriptionInput.focus()}
            placeholder="$0.00" 
            ref={(input) => this.priceInput = input}
            returnKeyType="done"
            value={this.state.price} />
        </View>
      </TouchableOpacity>
    );
  }

  /**
   * Renders the multiline description field for a user to enter extra
   * details regarding their listing.
   */
  renderDescriptionField = () => {
    let characterLimit = 250;

    return (
      <View
        backgroundColor="#FFF"
        padding={15}>
          
        <View
          flexDirection="row"
          justifyContent="space-between">
          
          <Text style={{fontSize: 14, fontWeight: '500'}}>Description</Text>
          <Text style={{color: '#C7C7CC', fontSize: 14, fontWeight: '500'}}>{characterLimit} limit</Text>
        </View>
      
        <TextInput
          blurOnSubmit={true}
          height={100}
          maxLength={characterLimit}
          multiline={true}
          numberOfLines={5}
          onChangeText={(text) => this.setState({description: text})}
          placeholder="Add any additional comments you want to share with other users..." 
          returnKeyType="done" 
          ref={(input) => this.descriptionInput = input}
          value={this.state.description} />
      </View>
    );
  }

  /**
   * Renders the default posting view.
   */
  renderDefaultView = () => {
    return (
      <View style={styles.container}>
        <KeyboardAwareScrollView extraHeight={150} enableOnAndroid={true}>
          { this.renderSearchField() }
          { this.renderSeparator("Begin by searching for a book or scanning a barcode by tapping the camera icon") }

          { this.renderPhotoUpload() }
          { this.renderSeparator() }

          { this.renderSelectDescriptor("Condition", "ConditionSelection") }
          { this.renderPriceDescriptor("Price", "decimal-pad") }
          { this.renderSeparator() }

          { this.renderDescriptionField() }
        </KeyboardAwareScrollView>
        
        {/* Post button */}
        <View style={{flexDirection: 'row'}}>
          <TouchableOpacity 
            onPress={() => this.resetState() }
            style={styles.cancelButton}>

            <Text style={styles.buttonText}>Clear</Text>
          </TouchableOpacity>
          <TouchableOpacity 
            onPress={() => this.attemptCreateListing() }
            style={styles.postButton}>

            <Text style={styles.buttonText}>Post</Text>
          </TouchableOpacity>
        </View>
      </View>
    );
  }

  /**
   * Renders an alternative view when user is searching for a book by title.
   * The normal view gets hidden and replaced with a search and autocomplete suggestions.
   */
  renderSearchView = () => {
    return (
      <View style={styles.container}>

        {/* Don't remove this keyboard view. If this doesn't match up with the above
            function, the keyboard will hide when transitioning views */}
        <KeyboardAwareScrollView keyboardShouldPersistTaps='always' extraHeight={150} enableOnAndroid={true}>
          { this.renderSearchField() }
          <Search searchTerm={this.state.title} onSelect={this.searchCallback} />
        </KeyboardAwareScrollView>
      </View>
    );
  }

  /**
   * Callback function used when searching for a book instead of using the barcode scanner.
   * @param data The book object returned by the {@link Search} component.
   */
  searchCallback = (data) => {
    this.setState({
      book: data,
      title: data.title,
      bookSelected: true
    });
  }

  render() {
    const isSearching = this.state.title.length > 0 && !this.state.bookSelected;    
    return isSearching ? this.renderSearchView() : this.renderDefaultView();
  }
}

const styles = StyleSheet.create({
  container: {
    backgroundColor: '#F6F6FB',
    flex: 1,
    overflow: 'hidden',
    paddingTop: 5
  },
  descriptorButton: {
    backgroundColor: '#FFF',
    padding: 15
  },
  cancelButton: {
    backgroundColor: '#f75555',
    padding: 30,
    width: "50%"
  },
  postButton: {
    backgroundColor: '#5694F7',
    padding: 30,
    width: "50%"
  },
  thumbnailButton: {
    backgroundColor: '#F6F6FB',
    borderRadius: 4,
    padding: 20
  },
  buttonText: {
    color: '#FFF',
    fontSize: 18,
    fontWeight: '500',
    textAlign: 'center'
  },
  input: {
    fontSize: 16,
    fontWeight: '500',
    height: 75,
    paddingHorizontal: 10,
    opacity: 1.0,
  },
  label: {
    color: '#464A53',
    fontWeight: '500',
    margin: 15,
    textAlign: 'center'
  },
});
