/**
 * Defines a Message (or a ConversationItem in the ConversationListView)
 * @author Ted Bieber, Stephen Andrews
 * @since 10.25.18
 */
import React, {
    Component
  } from 'react';
  import {
    ActivityIndicator,
    View,
    Text,
    StyleSheet,
    Image,
    TouchableOpacity,
  } from 'react-native';
  
  import Icon from 'react-native-vector-icons/Feather';

  import HorizontalRule from '../../components/HorizontalRule';
  import API from '../../API';

  import Alert from 'Alert';

  export default class Message extends Component {
  
    constructor() {
      super();

      this.state = {
        isLoading: true,
        read: "",
      }
    }

    componentDidMount() {
      this.setState({
        isLoading: false,
        isSender: this.props.user_id == (this.messageIsTradeRequest(this.props.item) ? this.props.item.buyer.user_id:this.props.item.sent_by)
      });
    }

    /**
     * Checks if the message is a trade request. Returns true or false
     * Takes in the item (this.props.item)
     * 
     * Currently, it checks if the item has a listing by checking if the listing has a book, and checking if the item has a buyer by checking buyer.user_id
     * If one of these is not defined, it will throw an exception, which will be caught and it will return false. 
     * 
     * The distinction between a message and a trade request can be viewed on the backend. 
     * A message will have the serialization of the message model, and a trade request/transaction will have the serialization of a transaction.
     */
    messageIsTradeRequest = (item) => {
      try {
        item.listing.book;
        item.buyer.user_id;
        return true;
      }
      catch(err) {
        return false;
      }
      
    }

    /**
     * Render a received message.
     * @return A view container a message sent by the current user.
     * 
     * This renders different looking and functioning components depending on whether the item is a trade request or a message,
     * If it's a message, go to the very bottom of the function. It simply renders the message
     * If it's a trade request, it will render in one of 4 different ways, depending on the status id of the message
     * Status codes are assumed to be: 1 = PENDING, 2 = DECLINED, 3 = ACCEPTED, 4 = CANCELED
     * Since it is rendering a message/trade request depending on whether the person is a sender or receiver, the phrasing of the component will depend on this.
     * There are two methods that this is split into: renderReceived() and renderSent().  
     *** Only the sender may cancel the trade request, and only the receiver can accept or decline it. These options are provided as a function of differing UI.
     *** As such, if the UI is somehow mistaking one user as a sender, then the user WILL be able to cancel the request.
     */
    renderReceived() {
      if(this.messageIsTradeRequest(this.props.item)) {
        return (
          <View style={[styles.receivedContainer]}>
            <View style={styles.receivedBubble}>
              { // Either it is pending or complete
                this.props.item.status == 1 ? 

                // Pending
                (<TouchableOpacity onPress={this.onPressSeller}>
                <Text>
                  { this.props.to_username + " has sent you a trade request:"}
                </Text>
                <Text style={styles.receivedText}>
                  { this.props.item.listing.book.title + " for $" + this.props.item.listing.price}
                </Text>
                
              </TouchableOpacity>)
              :
              (this.props.item.status == 4 ?
              // Canceled
              (<Text>
                <Text>
                  { this.props.to_username + " has canceled their trade request:\n"}
                </Text>
                <Text style={styles.receivedText}>
                  { this.props.item.listing.book.title + " for $" + this.props.item.listing.price}
                </Text>
                
              </Text>)
              :
                // Complete (Accepted or Declined)
              (<Text>
                <Text>
                  { "You have " + (this.props.item.status==3?"accepted ":"declined ") + this.props.to_username + "'s trade request:\n"}
                </Text>
                <Text style={styles.receivedText}>
                  { this.props.item.listing.book.title + " for $" + this.props.item.listing.price}
                </Text>
                
              </Text>)
              )
                
              }
            </View>
          </View>
          
        )
      }

      return (
        <View style={[styles.receivedContainer]}>
        {/* Above, we have an additional style added to reverse the rendering of the message bubble. This is because we inverted it in the ConversationListView */}
          <View style={styles.receivedBubble}>
            <Text style={styles.receivedText}>
            { this.props.item.message }
            </Text>
          </View>
        </View>
      );
    }

    /**
     * Render a sent message.
     * @return A view container a message sent by the current user.
     * 
     * This renders different looking and functioning components depending on whether the item is a trade request or a message,
     * If it's a message, go to the very bottom of the function. It simply renders the message
     * If it's a trade request, it will render in one of 4 different ways, depending on the status id of the message
     * Status codes are assumed to be: 1 = PENDING, 2 = DECLINED, 3 = ACCEPTED, 4 = CANCELED
     * Since it is rendering a message/trade request depending on whether the person is a sender or receiver, the phrasing of the component will depend on this.
     * There are two methods that this is split into: renderReceived() and renderSent().  
     *** Only the sender may cancel the trade request, and only the receiver can accept or decline it. These options are provided as a function of differing UI.
     *** As such, if the UI is somehow mistaking one user as a sender, then the user WILL be able to cancel the request.
     */
    renderSent() {
      if(this.messageIsTradeRequest(this.props.item)) {
        return (
          <View style={[styles.sentContainer]}>
            <View style={styles.sentBubble}>
            { // Either it is pending or complete
              this.props.item.status == 1 ?
              // Pending
              (
                <TouchableOpacity onPress={this.onPressBuyer}>
                  <View flexDirection="row" alignItems="center">
                    <View alignItems="center" justifyContent="center" style={{
                      backgroundColor: 'rgba(255, 255, 255, 0.2)',
                      borderRadius: 20,
                      width: 40,
                      height: 40,
                      marginRight: 5
                    }}>
                      <Icon name="package" size={25} color="#FFF"/>
                    </View>
                    
                    <View flexDirection="column">
                      <Text style={styles.sentText}>{"You have sent " + this.props.to_username + " a trade request"}</Text>
                      <Text style={[styles.sentText, {flex: 1, flexWrap: 'wrap'}]}>{ this.props.item.listing.book.title + " for $" + this.props.item.listing.price}</Text>
                    </View>
                  </View>
                  
                </TouchableOpacity>
              )
              :
              (this.props.item.status == 4 ?
              // Canceled
              (<Text>
                <Text>
                  {"You have canceled your trade request:\n"}
                </Text>
                <Text style={styles.sentText}>
                  { this.props.item.listing.book.title + " for $" + this.props.item.listing.price}
                </Text>
                
              </Text>)
              :
              // Complete (Accepted or Declined)
              (<Text>
                <Text>
                  {this.props.to_username + " has " + (this.props.item.status==3?"accepted":"declined") + " your trade request:\n"}
                </Text>
                <Text style={styles.sentText}>
                  { this.props.item.listing.book.title + " for $" + this.props.item.listing.price}
                </Text>
                
              </Text>)
              )
              
              
            }
            </View>
          </View>
            
        )
      }
      

      return (
        <View style={[styles.sentContainer]}>
        {/* Above, we have an additional style added to reverse the rendering of the message bubble. This is because we inverted it in the ConversationListView */}
          <View style={styles.sentBubble}>
            <Text style={styles.sentText}>
            { this.props.item.message }
            </Text>
          </View>
        </View>
      );
    }

    // The on-press trade request from the buyer's pespective
    onPressBuyer = () => {
      console.log(this.props.item);
      let listing = this.props.item.listing;
      Alert.alert(
        listing.book.title + " for $" + listing.price,
        'What would you like to do?',
        [
          {text: 'Nevermind', onPress: () => {console.log("Nevermind")}}, // DO API CALLS?
          {text: 'Cancel Trade Request', onPress: () => {this.onCancelBuyer()}}
        ],
        // { cancelable: false },
        {onDismiss:()=>{}}
      );
    }

    // The on-press trade request from the seller's pespective
    onPressSeller = () => {
      console.log(this.props.item);
      let listing = this.props.item.listing;
      Alert.alert(
        listing.book.title + " for $" + listing.price,
        'What would you like to do?',
        [
          {text: 'Decline', onPress: () => {this.onDeclineSeller()}}, // DO API CALLS?
          {text: 'Accept', onPress: () => {this.onAcceptSeller()}}
        ],
        // { cancelable: false },
        {onDismiss:()=>{}}
      );
    }

    onAcceptSeller = () => {
      let cb = (response) => {
        response.json().then((data)=> {
          console.log("Accepted the trade request and got json data: \n" + data);
          API.sendMessage(this.props.user_id, this.props.item.buyer.user_id, "I have accepted your trade request", (response) => {});
        }).catch((error)=>{
          console.log(error);
        });
       
      }
      API.sendTransaction(this.props.item.buyer.user_id, this.props.item.listing.listing_id, "COMPLETED", this.props.item.id, cb);
    }

    onDeclineSeller = () => {
      let cb = (response) => {
        response.json().then((data)=> {
          console.log("Declined the trade request and got json data: \n" + data);
          API.sendMessage(this.props.user_id, this.props.item.buyer.user_id, "I have declined your trade request", (response) => {});
        }).catch((error)=>{
          console.log(error);
        });
      }

      API.sendTransaction(this.props.item.buyer.user_id, this.props.item.listing.listing_id, "DECLINED", this.props.item.id, cb);
    }

    onCancelBuyer = () => {
      let cb = (response) => {
        response.json().then((data)=> {
          console.log("Canceled the trade request and got json data: \n" + data);
          API.sendMessage(this.props.user_id, this.props.item.buyer.user_id, "I have canceled my trade request", (response) => {});
        }).catch((error)=>{
          console.log(error);
        });
      }

      API.sendTransaction(this.props.item.buyer.user_id, this.props.item.listing.listing_id, "CANCELED", this.props.item.id, cb);
    }

    render() {
      return this.state.isSender ? this.renderSent() : this.renderReceived();
    }
  }

  const styles = StyleSheet.create({
    receivedContainer: {
      backgroundColor: '#FDFDFE',
      flex: 1,
      flexDirection: 'row',
      justifyContent: 'flex-start',
      paddingHorizontal: 10,
      paddingVertical: 5
    },
    receivedBubble: {
      backgroundColor: '#E5E5EA',
      borderWidth: 1,
      borderColor: '#DDDDDD',
      borderRadius: 20,
      maxWidth: '75%',
      padding: 10
    },
    receivedText: {
      color: 'black'
    },
    sentContainer: {
      backgroundColor: '#FDFDFE',
      flex: 1,
      flexDirection: 'row',
      justifyContent: 'flex-end',
      paddingHorizontal: 10,
      paddingVertical: 5
    },
    sentBubble: {
      backgroundColor: '#4291F2',
      borderWidth: 1,
      borderColor: '#DDDDDD',
      borderRadius: 20,
      maxWidth: '75%',
      padding: 10
    },
    sentText: {
      color: '#FFFFFF',
    },
  });
  