import React, {
  Component
} from 'react';
import {
  Platform,
  View,
  Text,
  StatusBar,
  StyleSheet,
  TouchableOpacity,
  Image, ScrollView
} from 'react-native';

import {NavigationActions, StackActions} from 'react-navigation';

import Icon from 'react-native-vector-icons/Feather';

import API from '../../API';
import BookShelf from '../bookshelf/BookShelf';
import * as Constants from '../../Constants';
import HorizontalRule from '../../components/HorizontalRule';
import SplitView from '../../components/SplitView';
import StorageManager from '../../StorageManager';
import ChangePasswordModal from './ChangePasswordModal';
import BugReportModal from './BugReportModal';

/**
 * User's profile screen for managing their account.
 * @author Stephen Andrews, John Bieber
 */
export default class Profile extends Component {

  constructor() {
    super();

    this.state = {
      username: "",
      email: "",
      university_id: "",
      user_id: -1,
      profile_picture: "",
      bugReportModalVisible: false,
      changePasswordModalVisible: false
    };

  }

  componentDidMount() {
    this.loadUserData();
  }

  componentWillUpdate() {
    StorageManager.getItem('user_id', (value) => {
      if (this.state.user_id != value) {
        this.loadUserData();  
      }
    });

    if (this.props.navigation.state.params) {
      if (this.props.navigation.state.params.bookId) {
        this.props.navigation.state.params = {};
        this.loadUserData();
      }
    }
  }

  /**
   * Check to see if the user scanned a book with the barcode scanner.
   * If scanned, the state is updated with the data for that book.
   */
  componentDidUpdate() {
    // Determines if a child component returned data via navigation
    let hasReceivedData = this.props.navigation.state.params != null;
    if (!hasReceivedData) {
      return;
    }

    // User photos
    let photoTaken = this.props.navigation.state.params.photo;
    if (photoTaken) {
      this.props.navigation.state.params.photo = null;
      let temp = {... this.state};
      //////////
      temp['profile_picture'] = photoTaken.uri;
      this.setState({
        ... temp
      });
      StorageManager.getItem('username', (username) => {
        API.uploadPhotos(
          [photoTaken],
          username,
          false,
          (response) => {
            if (response.status == 200) {
            }
          },
          (url) => {
            API.updateProfilePicture(
              username,
              url[0],
              (response) => {
                if (response.status == 200) {
                  StorageManager.putItem("profile_picture", url[0]);
                }
              }
            );
          }
        );
      }, () => {
        console.log("Failed to load username from storage manager.");
      });

      //////////
    }
  }

  /**
   * Load user data from the {@link StorageManager}.
   */
  loadUserData = () => {
    Object.keys(this.state).forEach((key) => {

      // Load global props from storage manager
      StorageManager.getItem(key, (value) => {
        // Can't use setState with variable name key
        this.state[key] = value;
        this.forceUpdate();
      });
    });
  }

  onPressLogout = () => {
    StorageManager.putItem("remember_me", false);
    this.props.navigation.navigate('LoadingScreen');
  }

  /**
   * Render the users profile picture. Pressing on the picture
   * will allow the user to change their profile picture.
   */
  renderProfilePicture = () => {
    let defaultPicture = require('../../assets/noprofilepic.png');
    let hasPicture = this.state.profile_picture != null;
    let picture = hasPicture ? (this.state.profile_picture ? {uri: this.state.profile_picture} : defaultPicture) : defaultPicture; 

    return (
      <TouchableOpacity 
        onPress={() => this.props.navigation.navigate('CameraView', {returnScreen: 'Profile'})}
        style={{
          backgroundColor: '#FFF',
          borderRadius: 80,
          padding: 5,
          margin: 5,
          shadowColor: 'black',
          shadowOffset: {width: 0, height: 0},
          shadowRadius: 5,
          shadowOpacity: 0.20,
          elevation: 5
        }}>

        <Image
          source={picture}
          style={{
            borderRadius: 75,
            width: 150,
            height: 150
          }} />
      </TouchableOpacity>
    );
  }

  /**
   * Render the user's profile containing their picture, name, and username.
   */
  renderProfileContent = () => {
    return (
      <View alignItems="center">
        { this.renderProfilePicture() }

        <View 
          alignItems="center" 
          marginVertical={15}>
          
          {/* <Text style={styles.heading}>Stephen Andrews</Text> */}
          <Text style={styles.subHeading}>@{this.state.username}</Text>
        </View>
      </View>
    ); 
  }


  /**
   * Render an option pertaining to the user's profile.
   * @param title The name of the option.
   * @param description A detailed description of the option.
   * @param onPress Action to perform when option is pressed.
   * @param hasChild An optional parameter to indicate whether or not the option has a child view.
   *                 When set to true, a chevron-right is displayed. Defaults to true.
   */
  renderOption = (title, description, onPress, hasChild=true) => {
    let chevron = <Icon style={{marginTop: -2}} name="chevron-right" color="#C7C7CC" size={20} />;

    return (
      <View>
        <View 
          alignItems="center"
          flexDirection="row"
          justifyContent="space-between">
          
          <TouchableOpacity 
            onPress={() => onPress()}
            style={{flex: 1, padding: 10}}>      
            <Text style={{
              fontSize: 18,
              marginBottom: 5
            }}>{title}</Text>
            <Text style={{color: '#CDCFD1'}}>{description}</Text>
          </TouchableOpacity>

          {hasChild ? chevron : <View></View>}
        </View>
        
        <HorizontalRule />
      </View>
    );
  }

  dismissBugReportModal = () => {
    this.setState({
      ...this.state, 
      'bugReportModalVisible': false
    });
  }
  
  dismissPasswordModal = () => {
    this.setState({
      ...this.state, 
      'changePasswordModalVisible': false
    });
  }

  render() {
    return (
      <View style={styles.container}>
        <StatusBar barStyle="dark-content" /> 
        <BugReportModal
          state={{...this.state}}
          onPress={this.dismissBugReportModal}
          visible={this.state.bugReportModalVisible}
          {...this.props} />
        <ChangePasswordModal
            state={{...this.state}}
            onPress={this.dismissPasswordModal}
            visible={this.state.changePasswordModalVisible} 
            {...this.props} />
          
        <ScrollView showsVerticalScrollIndicator={false}>
          { this.renderProfileContent() }

          {/* { this.renderOption('Notifications', 'Edit notification settings', () => {}) } */}
          { this.renderOption('Active listings', 'View all of your active listings', () => {this.props.navigation.navigate('ActiveListings')}) }
          { this.renderOption('Transaction history', 'Get a detailed view of all your transactions', () => {this.props.navigation.navigate('TransactionHistory')}) }
          {/* { this.renderOption('Get help', 'Reference material for our app', () => {}) } */}
          { this.renderOption('Bug Report', 'Report a bug that needs fixing', () => {this.setState({'bugReportModalVisible': true});}) }
          { this.renderOption('Change Password', 'Permanently change your password', () => {this.setState({'changePasswordModalVisible': true});}) }
          { this.renderOption('Log out', 'Terminates your current session', this.onPressLogout, false) }
          {/* <BookShelf {...this.props} /> */}
        </ScrollView>
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    backgroundColor: '#FDFDFE',
    flex: 1,
    paddingTop: 70,
    paddingHorizontal: 20
  },
  heading: {
    color: '#464A53',
    fontSize: 22,
    fontFamily: Platform.OS === 'ios' ? 'Helvetica' : 'Roboto',
    fontWeight: '500',
    padding: 5
  },
  subHeading: {
    color: Constants.PRIMARY_BLUE,
    fontSize: 16,
    fontFamily: Platform.OS === 'ios' ? 'Helvetica' : 'Roboto',
  },
  buttonContainer: {
    backgroundColor: '#4392EF',
    borderRadius: 4,
    height: 50,
    paddingVertical: 12
  },
  buttonText: {
    color: '#FFF',
    fontSize: 18,
    fontWeight: '500',
    textAlign: 'center'
  },
});
